/* This file is released into the public domain by its authors */

#include <stdio.h>
#include <stdlib.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <time.h>
#include <sys/poll.h>
#include <sys/time.h>
#include <string.h>
#include <limits.h>
#include "dvbdemux.c"

#define BSIZE 188

static int pidt[0x2001];


FILE* settingsfile;
FILE* outfile;

time_t t;
struct tm *tmp;

long i,ok,tzapok,rmins,rminstop,istop;

int tzapstatus,sysreturn,tzap_was_running;

double pscale1,pscale2,tscale,pt1,pt2,rstopms;

char outdirname[1024],appdirname[1024],settingsfilename[1024],outfilename[1024];
char timestring[64],tzapfilename[1024],chname[64],listname[64],commandstring[1024];
char charzero = (char)0;

int read_settingsfile(void);
void delay(long);

int main(int argc, char **argv)
{
  struct timeval startt,runstartt;
  int adapter = 0, demux = 0;
  char *search = NULL;
  int fd, ffd, packets = 0;
  int opt;
  
  istop=0;
  
  pscale1= 188.0/1024.0; pscale2=8.0*188.0/1000.0;
  
  printf("  ***DVB Traffic Logger V1.01*** J. Lesurf  6th Oct 2012\n");
  printf("  ======================================================\n\n");
  printf("  Records DVB packet stats and signal conditions.\n\n");
  sprintf(appdirname,"%s",argv[0]);
  i=(int)strrchr(appdirname,'/')-(int)appdirname;
  appdirname[i]=charzero;
  printf("Application Dir at = %s\n\n",appdirname);
  sprintf(settingsfilename,"%s/Settings\0",appdirname);
  printf("Settings file = %s\n",settingsfilename);
  ok=read_settingsfile();
  
  sysreturn=system("pgrep '^tzap$' > /dev/null 2>&1");
  tzapstatus=WEXITSTATUS(sysreturn);
  printf("tzap status = %d\n",tzapstatus);
  
  if(tzapstatus==0)
  {
    printf("tzap already running\n");
    tzap_was_running=1;
  }
  else
  {
    tzap_was_running=0;
    t=time(NULL);
    tmp=localtime(&t);
    strftime(timestring,sizeof(timestring),"%a%d%b%Hh%Mm%Ss\0",tmp);
    printf("Time = %s\n",timestring);
    printf("\ntzap not running!\n");
    
    snprintf (commandstring, sizeof(commandstring),"xterm -fn 10x20 -sb -rightbar -sl 200 -e ""sh -c \"exec tzap -rpc %s '%s' 2>&1 | tee %s/%s_SignalMonitor\" &",listname, chname, outdirname, timestring);

    printf("Tzap Command = %s\n",commandstring);
    system(commandstring);
    delay(300);       /* This line sets a delay between the tzap and the start of logging to give the tuner a chance to settle and lock. */
    printf("Started tzap\n");
  }
  
  t=time(NULL);
  tmp=localtime(&t);
  strftime(timestring,sizeof(timestring),"%a%d%b%Hh%Mm%Ss\0",tmp);
  printf("Time = %s\n",timestring);
  
  sprintf(outfilename,"%s/%s_for_%02ldmins_TrafficLog\0",outdirname,timestring,rminstop);
  printf("Output to file %s\n",outfilename);
  printf("Stop after %ld mins\n",rminstop);
  rstopms=60000*rminstop;
  outfile=fopen(outfilename,"wt");
  
  printf("\nPlease wait. First 5-sec block being counted.\n\n");

  // open the DVR device
  fd = dvbdemux_open_dvr(adapter, demux, 1, 0);
  if (fd < 0) {
     fprintf(stderr, "dvbtraffic: Could not open dvr device: %m\n");
    exit(1);
  }
  dvbdemux_set_buffer(fd, 1024 * 1024);

  ffd = dvbdemux_open_demux(adapter, demux, 0);
  if (ffd < 0) {
    fprintf(stderr, "dvbtraffic: Could not open demux device: %m\n");
    exit(1);
  }

  if (dvbdemux_set_pid_filter(ffd, -1, DVBDEMUX_INPUT_FRONTEND, DVBDEMUX_OUTPUT_DVR, 1)) {
    perror("dvbdemux_set_pid_filter");
    return -1;
  }

  gettimeofday(&startt, 0);
  runstartt=startt;

  while (istop==0) {
    unsigned char buffer[BSIZE];
    int pid, ok;
    ssize_t r;

    if ((r = read(fd, buffer, BSIZE)) <= 0) {
      perror("read");
      break;
    }
    if (r != BSIZE) {
      fprintf(stderr, "dvbtraffic: only read %zd bytes\n", r);
      break;
    }
    if (buffer[0] != 0x47) {
      continue;
      printf("desync (%x)\n", buffer[0]);
      while (buffer[0] != 0x47)
        read(fd, buffer, 1);
      continue;
    }
    ok = 1;
    pid = ((((unsigned) buffer[1]) << 8) |
           ((unsigned) buffer[2])) & 0x1FFF;

    if (search) {
      int i, sl = strlen(search);
      ok = 0;
      if (pid != 0x1fff) {
        for (i = 0; i < (188 - sl); ++i) {
          if (!memcmp(buffer + i, search, sl))
            ok = 1;
        }
      }
    }

    if (ok) {
      pidt[pid]++;
      pidt[0x2000]++;
    }

    packets++;

    if (!(packets & 0x7F)) {
      struct timeval now;
      long diff,rundiff;
      gettimeofday(&now, 0);
      diff =
          (now.tv_sec - startt.tv_sec) * 1000 +
          (now.tv_usec - startt.tv_usec) / 1000;
      rundiff =
          (now.tv_sec - runstartt.tv_sec) * 1000 +
          (now.tv_usec - runstartt.tv_usec) / 1000;   
          
      if (diff > 4995) {
        int _pid = 0;
        tscale= 1000.0 / (double)diff;
        /* pt1=pscale1*tscale; */
        pt2=pscale2*tscale;
        for (_pid = 0; _pid < 0x2001; _pid++) {
          if (pidt[_pid]) {
            printf("%04x %8.2lf p/s  %9.2lf kbps\n",_pid,tscale*(double)pidt[_pid],pt2*(double)pidt[_pid]);
            fprintf(outfile,"%04x %8.2lf p/s  %9.2lf kbps\n",_pid,tscale*(double)pidt[_pid],pt2*(double)pidt[_pid]);
          }
          pidt[_pid] = 0;
        }
        printf("-PID--PACKETS--------BANDWIDTH---- dt = %5ld [ %7.4lf ]  %9.3lf\n",diff,tscale,0.001*(double)rundiff);
        fprintf(outfile,"-PID--PACKETS--------BANDWIDTH---- dt = %5ld [ %7.4lf ]  %9.3lf\n",diff,tscale,0.001*(double)rundiff);
        startt = now;
        
        if(rundiff>=rstopms)
        {
          istop=1;
          printf("Run finished OK\n");
        }
        
      }
    }
  }
  fclose(outfile);
  close(ffd);
  close(fd);
  
  if(tzap_was_running==1)
  {
    printf("tzap was running *before* traffic recording started.\n  Do you wish tzap to stop?\n");
    printf(" Enter '0' to kill tzap - or '1' for it to continue\n  => ");
    scanf("%d",&tzap_was_running);
  }
  
  if (tzap_was_running==0)
  {
    printf("Killing tzap process\n");
    system("pkill '^tzap$'");
  }
  printf("\n  **Done!**\n");
  return 0;
}

void delay(long dtime)
{
  long td0,td1,tdum,tdel;
  tdel=0;
  td0=(long)clock();
  td0=(100*td0)/CLOCKS_PER_SEC;
  do
  {
    tdum=100/22.0;
    td1=(long)clock();
    td1=(100*td1)/CLOCKS_PER_SEC;
  } while ( (td1-td0)<dtime);
  printf("Delay = %ld centisec\n",td1);
}

int read_settingsfile(void)
{
   char tempname[128],ch0;
   int rok,lok,chin;
   settingsfile=fopen(settingsfilename,"rt");

   if((int)settingsfile>0)
   {
     printf("\nReading settings ok\n");
     rok=0;
     lok=(int)fscanf(settingsfile,"%s",outdirname);
     if (lok>0) rok=1;
     lok=(int)fscanf(settingsfile,"%ld",&rminstop);
     
     printf("Output directory name = %s\n",outdirname);
     fscanf(settingsfile,"%c",&ch0);
     chin=0;
     do
     {
      fscanf(settingsfile,"%c",&ch0);
      if(14<(int)ch0)
      {
        chname[chin]=ch0;
        chin++;
      }
      
     } while (14<(int)ch0);
     chname[chin]=0;
     
     printf("Nominal station = %s\n",chname);
     fscanf(settingsfile,"%s",tempname);
     sprintf(listname,"%s/tzap_mux/%s\0",appdirname,tempname);
     printf("List file name = %s\n",listname);
     
   }
   else
   {
     printf("no settings file!\n");
     rok=0;
   }
   fclose(settingsfile);
   return rok;
}
